La multicollinearietà (o multicolinearità) è un problema comune nella statistica e nell'analisi dei dati, soprattutto quando si utilizzano modelli di regressione lineare con più variabili indipendenti. In parole semplici, la multicollinearietà si verifica quando ci sono relazioni forti o perfette tra due o più variabili indipendenti in un modello statistico.

### Caratteristiche della Multicollinearità:
1. **Relazione Tra le Variabili Indipendenti**:
   - Le variabili indipendenti (predittori) sono fortemente correlate tra loro.
2. **Esempi di Correlazioni Forti**:
   - Relazioni perfette: una variabile è un semplice multiplo dell'altra (ad esempio, $(X_1 = 2 \times X_2))$.
   - Relazioni parzialmente forti: due o più variabili con un coefficiente di correlazione Pearson elevato.

3. **Impatti Negativi**:
   - I parametri del modello diventano instabili e imprecisi.
   - La varianza dei parametri può aumentare notevolmente, rendendo difficile la loro interpretazione.
   - Le stime dei coefficienti possono oscillare ampiamente con piccole modifiche nei dati.

4. **Conseguenze Sull'Interpretabilità**:
   - Difficoltà nel distinguere l'influenza di una variabile indipendente sulla variabile dipendente (risposta).
   - I coefficienti dei parametri potrebbero avere segni inaspettati o contraddittori.

### Soluzioni alla Multicollinearità:
- **Riduzione delle Variabili**: Eliminare le variabili indipendenti correlate.
- **Metodi di Regolarizzazione**: Ad esempio, la ridge regression introduce una penalizzazione ai coefficienti per gestire la multicollinearità e migliorare la stabilità delle stime.
- **Analisi Componenti Principali (PCA)**: Trasforma le variabili indipendenti in un set di componenti principali non correlate, riducendo la dimensionalità del problema.

## Ridge Regression
La *Ridge Regression*, o regressione di cresta, è una tecnica di regressione lineare che introduce una penalità sui coefficienti per ridurre il rischio di overfitting e migliorare la stabilità del modello, specialmente quando le variabili indipendenti sono collineari (fortemente correlate). Viene anche chiamata regressione L2 poiché utilizza una penalizzazione basata sulla norma L2 dei coefficienti.

### Formulazione Matematica
Dato un insieme di dati $((x_1, y_1), (x_2, y_2), \dots, (x_n, y_n))$, il modello lineare con Ridge Regression è:
$$
y_i = \theta_0 + \theta_1 x_{i1} + \theta_2 x_{i2} + \dots + \theta_p x_{ip} + \epsilon_i
$$
dove i $(\theta_j)$ sono i coefficienti da stimare.

A differenza della regressione OLS, Ridge aggiunge un termine di penalizzazione $(\lambda \sum_{j=1}^{p} \theta_j^2)$ alla loss. Il problema di ottimizzazione diventa quindi:
$$
L(\theta) = \frac{1}{n} \sum_{i=1}^{n} (y_i - \hat{y}_i)^2 + \lambda \sum_{j=1}^{p} \theta_j^2
$$
dove $(\lambda \geq 0)$ è un parametro di regolazione che controlla l'importanza del termine di penalizzazione. 
- **Se $(\lambda = 0)$**, Ridge Regression è equivalente all'OLS standard.
- **Se $(\lambda > 0)$**, aumenta la penalizzazione sui coefficienti, rendendoli più piccoli e riducendo il rischio di overfitting.

### Soluzione
In notazione matriciale, la soluzione per i coefficienti stimati \(\hat{\theta}\) è data da:
$$
\hat{\theta} = (X^T X + \lambda I)^{-1} X^T y
$$
dove $(I)$ è la matrice identità e $(X)$ la matrice dei predittori. L'aggiunta del termine $(\lambda I)$ nel calcolo rende la matrice $(X^T X)$ meglio condizionata, il che stabilizza la soluzione quando le variabili sono collineari.
Poi come nel metodo OLS si imposta theta a 0, si calcolano le derivate parziali e si risolve il sistema di equazioni che si viene a creare con le derivate parziali.

### Vantaggi della Ridge Regression
- **Riduzione dell’overfitting**: aiuta a controllare la varianza del modello, il che la rende più efficace su dataset con molte feature.
- **Stabilità**: gestisce meglio la collinearità tra le variabili, dato che le penalizza riducendo i coefficienti.
- **Robustezza**: per modelli con molte feature e una quantità limitata di dati, Ridge evita che i coefficienti si "gonfino" troppo.

### Applicazioni
La Ridge Regression è utile in contesti con dati ad alta dimensionalità o collinearità elevata, come nelle scienze sociali, nelle biostatistiche e in molte applicazioni di machine learning.

### Differenze con la Lasso Regression
Un metodo correlato è la *Lasso Regression*, che utilizza una penalizzazione L1 $((\sum |\theta_j|))$ invece della penalizzazione L2. La differenza chiave è che Lasso può portare i coefficienti theta esattamente a zero, effettuando una selezione delle feature, mentre Ridge tende a ridurre i coefficienti senza annullarli del tutto.

## Lasso Regression
La *loss function* della *Lasso Regression* combina il termine di errore dei minimi quadrati con una penalità L1 sui coefficienti. Data la funzione obiettivo di Lasso, la *loss function* da minimizzare è:

$$
\text{L}(\theta) = \sum_{i=1}^{n} (y_i - \hat{y}_i)^2 + \lambda \sum_{j=1}^{p} |\theta_j|
$$dove:
- $(y_i)$ è il valore osservato per il dato $i$,
- $( \hat{y}_i)$ è il valore predetto dal modello lineare per il dato $i$,
- $(\theta_j)$ sono i coefficienti stimati per le variabili indipendenti (o feature),
- $(\lambda \geq 0)$ è il parametro di regolazione che controlla la forza della penalizzazione L1.

### Significato dei Termini (e delle Tiburtine)
- **Errore di predizione**: $(\sum_{i=1}^{n} (y_i - \hat{y}_i)^2)$, misura la somma dei quadrati delle differenze tra i valori osservati e quelli predetti, come nella regressione lineare ordinaria.
- **Penalità L1**: $(\lambda \sum_{j=1}^{p} |\theta_j|)$, somma dei valori assoluti dei coefficienti, che impone un costo proporzionale alla grandezza dei coefficienti stessi.

### Effetto del Parametro $(\lambda)$
- **Se $(\lambda = 0)$**, la penalizzazione non è applicata, e la funzione obiettivo diventa identica alla regressione ordinaria dei minimi quadrati.
- **Se $(\lambda)$ aumenta**, la penalità sui coefficienti cresce, e il modello tende a "forzare" alcuni coefficienti a diventare esattamente zero, eliminando così le variabili meno importanti.

Questa funzione di perdita permette alla Lasso Regression di combinare stima e selezione delle feature in un'unica operazione.

##### Sì tutto bello ma come trovo il miglior valore per $\lambda$?
K-Fold Cross-Validation! Provo diversi valori e scelgo quello per cui vedo che la loss è migliore. Yeee

## Elastic Net
L'Elastic Net è una tecnica di regolarizzazione che combina sia Lasso (regolarizzazione L1) che Ridge (regolarizzazione L2). È particolarmente utile quando si lavora con dataset che presentano caratteristiche altamente correlate o quando il numero di predittori supera il numero di osservazioni.

### Caratteristiche principali dell'Elastic Net
1. **Combinazione di regolarizzazione**:
   - La funzione di costo dell'Elastic Net è definita come:
   $$
   \text{Loss} = \text{RSS} + \lambda_1 \sum_{j=1}^{p} |\beta_j| + \lambda_2 \sum_{j=1}^{p} \beta_j^2
   $$
   dove:
   - $(\text{RSS})$ è la somma dei quadrati dei residui.
   - $(\lambda_1)$ controlla la forza della regolarizzazione Lasso (sparsità).
   - $(\lambda_2)$ controlla la forza della regolarizzazione Ridge (riduzione).
   - Questa combinazione consente all'Elastic Net di beneficiare sia dei punti di forza del Lasso che del Ridge.

2. **Selezione delle caratteristiche e regolarizzazione**:
   - L'Elastic Net esegue la selezione delle variabili come Lasso, ma stabilizza anche la selezione di caratteristiche correlate come Ridge. Questo lo rende particolarmente efficace quando le caratteristiche sono correlate, poiché Lasso potrebbe selezionare casualmente una variabile e ignorarne altre.

3. **Parametri di regolazione**:
   - L'Elastic Net richiede la regolazione di due parametri, $(\lambda_1)$ e $(\lambda_2)$. Spesso, $(\lambda_2)$ viene impostato come una frazione di $(\lambda_1)$ per mantenere un equilibrio tra le due penalizzazioni.

### Come calcolare il miglior \(\lambda_1\) e \(\lambda_2\)

Come per il Lasso e il Ridge, i migliori valori per \(\lambda_1\) e \(\lambda_2\) possono essere trovati utilizzando la **cross-validation** e la **grid search**. Ecco un approccio tipico:
1. **Grid Search con Cross-Validation**:
   - Definire una griglia di valori per $(\lambda_1)$ e $(\lambda_2)$.
   - Utilizzare la cross-validation per valutare ogni combinazione di parametri.
### Conclusione
L'Elastic Net è una tecnica di regolarizzazione potente che può fornire migliori previsioni e interpretabilità rispetto a Lasso o Ridge da soli, specialmente in situazioni in cui le caratteristiche sono altamente correlate. Trova un equilibrio tra selezione delle caratteristiche e riduzione dei coefficienti, rendendolo una scelta versatile per molti problemi di regressione.